@@ -2,8 +2,9 @@
 
 """
 
-from Tkinter import *
+from tkinter import *
 import os
+import sys
 
 from idlelib import textView
 from idlelib import idlever
@@ -12,11 +13,16 @@ class AboutDialog(Toplevel):
     """Modal about dialog for idle
 
     """
-    def __init__(self, parent, title):
+    def __init__(self, parent, title, _htest=False):
+        """
+        _htest - bool, change box location when running htest
+        """
         Toplevel.__init__(self, parent)
         self.configure(borderwidth=5)
-        self.geometry("+%d+%d" % (parent.winfo_rootx()+30,
-                                  parent.winfo_rooty()+30))
+        # place dialog below parent if running htest
+        self.geometry("+%d+%d" % (
+                        parent.winfo_rootx()+30,
+                        parent.winfo_rooty()+(30 if not _htest else 100)))
         self.bg = "#707070"
         self.fg = "#ffffff"
         self.CreateWidgets()
@@ -57,7 +63,8 @@ class AboutDialog(Toplevel):
                            justify=LEFT, fg=self.fg, bg=self.bg)
         labelEmail.grid(row=6, column=0, columnspan=2,
                         sticky=W, padx=10, pady=0)
-        labelWWW = Label(frameBg, text='www:  http://www.python.org/idle/',
+        labelWWW = Label(frameBg, text='https://docs.python.org/' +
+                         sys.version[:3] + '/library/idle.html',
                          justify=LEFT, fg=self.fg, bg=self.bg)
         labelWWW.grid(row=7, column=0, columnspan=2, sticky=W, padx=10, pady=0)
         Frame(frameBg, borderwidth=1, relief=SUNKEN,
